@@ -5,23 +5,26 @@
 #include "src/compiler-dispatcher/optimizing-compile-dispatcher.h"
 
 #include "src/base/atomicops.h"
-#include "src/cancelable-task.h"
-#include "src/compilation-info.h"
-#include "src/compiler.h"
-#include "src/isolate.h"
-#include "src/objects-inl.h"
+#include "src/codegen/compiler.h"
+#include "src/codegen/optimized-compilation-info.h"
+#include "src/execution/isolate.h"
+#include "src/init/v8.h"
+#include "src/logging/counters.h"
+#include "src/logging/log.h"
+#include "src/objects/objects-inl.h"
+#include "src/tasks/cancelable-task.h"
 #include "src/tracing/trace-event.h"
-#include "src/v8.h"
 
 namespace v8 {
 namespace internal {
 
 namespace {
 
-void DisposeCompilationJob(CompilationJob* job, bool restore_function_code) {
+void DisposeCompilationJob(OptimizedCompilationJob* job,
+                           bool restore_function_code) {
   if (restore_function_code) {
     Handle<JSFunction> function = job->compilation_info()->closure();
-    function->set_code(function->shared()->code());
+    function->set_code(function->shared().GetCode());
     if (function->IsInOptimizationQueue()) {
       function->ClearOptimizationMarker();
     }
@@ -39,12 +42,16 @@ class OptimizingCompileDispatcher::CompileTask : public CancelableTask {
  public:
   explicit CompileTask(Isolate* isolate,
                        OptimizingCompileDispatcher* dispatcher)
-      : CancelableTask(isolate), isolate_(isolate), dispatcher_(dispatcher) {
-    base::LockGuard<base::Mutex> lock_guard(&dispatcher_->ref_count_mutex_);
+      : CancelableTask(isolate),
+        isolate_(isolate),
+        worker_thread_runtime_call_stats_(
+            isolate->counters()->worker_thread_runtime_call_stats()),
+        dispatcher_(dispatcher) {
+    base::MutexGuard lock_guard(&dispatcher_->ref_count_mutex_);
     ++dispatcher_->ref_count_;
   }
 
-  virtual ~CompileTask() {}
+  ~CompileTask() override = default;
 
  private:
   // v8::Task overrides.
@@ -54,20 +61,26 @@ class OptimizingCompileDispatcher::CompileTask : public CancelableTask {
     DisallowHandleDereference no_deref;
 
     {
-      TimerEventScope<TimerEventRecompileConcurrent> timer(isolate_);
+      WorkerThreadRuntimeCallStatsScope runtime_call_stats_scope(
+          worker_thread_runtime_call_stats_);
+      RuntimeCallTimerScope runtimeTimer(
+          runtime_call_stats_scope.Get(),
+          RuntimeCallCounterId::kOptimizeBackgroundDispatcherJob);
 
+      TimerEventScope<TimerEventRecompileConcurrent> timer(isolate_);
       TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("v8.compile"),
-                   "V8.RecompileConcurrent");
+                   "V8.OptimizeBackground");
 
       if (dispatcher_->recompilation_delay_ != 0) {
         base::OS::Sleep(base::TimeDelta::FromMilliseconds(
             dispatcher_->recompilation_delay_));
       }
 
-      dispatcher_->CompileNext(dispatcher_->NextInput(true));
+      dispatcher_->CompileNext(dispatcher_->NextInput(true),
+                               runtime_call_stats_scope.Get());
     }
     {
-      base::LockGuard<base::Mutex> lock_guard(&dispatcher_->ref_count_mutex_);
+      base::MutexGuard lock_guard(&dispatcher_->ref_count_mutex_);
       if (--dispatcher_->ref_count_ == 0) {
         dispatcher_->ref_count_zero_.NotifyOne();
       }
@@ -75,6 +88,7 @@ class OptimizingCompileDispatcher::CompileTask : public CancelableTask {
   }
 
   Isolate* isolate_;
+  WorkerThreadRuntimeCallStats* worker_thread_runtime_call_stats_;
   OptimizingCompileDispatcher* dispatcher_;
 
   DISALLOW_COPY_AND_ASSIGN(CompileTask);
@@ -83,7 +97,7 @@ class OptimizingCompileDispatcher::CompileTask : public CancelableTask {
 OptimizingCompileDispatcher::~OptimizingCompileDispatcher() {
 #ifdef DEBUG
   {
-    base::LockGuard<base::Mutex> lock_guard(&ref_count_mutex_);
+    base::MutexGuard lock_guard(&ref_count_mutex_);
     DCHECK_EQ(0, ref_count_);
   }
 #endif
@@ -91,15 +105,16 @@ OptimizingCompileDispatcher::~OptimizingCompileDispatcher() {
   DeleteArray(input_queue_);
 }
 
-CompilationJob* OptimizingCompileDispatcher::NextInput(bool check_if_flushing) {
-  base::LockGuard<base::Mutex> access_input_queue_(&input_queue_mutex_);
+OptimizedCompilationJob* OptimizingCompileDispatcher::NextInput(
+    bool check_if_flushing) {
+  base::MutexGuard access_input_queue_(&input_queue_mutex_);
   if (input_queue_length_ == 0) return nullptr;
-  CompilationJob* job = input_queue_[InputQueueIndex(0)];
+  OptimizedCompilationJob* job = input_queue_[InputQueueIndex(0)];
   DCHECK_NOT_NULL(job);
   input_queue_shift_ = InputQueueIndex(1);
   input_queue_length_--;
   if (check_if_flushing) {
-    if (static_cast<ModeFlag>(base::Acquire_Load(&mode_)) == FLUSH) {
+    if (mode_ == FLUSH) {
       AllowHandleDereference allow_handle_dereference;
       DisposeCompilationJob(job, true);
       return nullptr;
@@ -108,26 +123,30 @@ CompilationJob* OptimizingCompileDispatcher::NextInput(bool check_if_flushing) {
   return job;
 }
 
-void OptimizingCompileDispatcher::CompileNext(CompilationJob* job) {
+void OptimizingCompileDispatcher::CompileNext(OptimizedCompilationJob* job,
+                                              RuntimeCallStats* stats) {
   if (!job) return;
 
   // The function may have already been optimized by OSR.  Simply continue.
-  CompilationJob::Status status = job->ExecuteJob();
+  CompilationJob::Status status = job->ExecuteJob(stats);
   USE(status);  // Prevent an unused-variable error.
 
-  // The function may have already been optimized by OSR.  Simply continue.
-  // Use a mutex to make sure that functions marked for install
-  // are always also queued.
-  base::LockGuard<base::Mutex> access_output_queue_(&output_queue_mutex_);
-  output_queue_.push(job);
+  {
+    // The function may have already been optimized by OSR.  Simply continue.
+    // Use a mutex to make sure that functions marked for install
+    // are always also queued.
+    base::MutexGuard access_output_queue_(&output_queue_mutex_);
+    output_queue_.push(job);
+  }
+
   isolate_->stack_guard()->RequestInstallCode();
 }
 
 void OptimizingCompileDispatcher::FlushOutputQueue(bool restore_function_code) {
   for (;;) {
-    CompilationJob* job = nullptr;
+    OptimizedCompilationJob* job = nullptr;
     {
-      base::LockGuard<base::Mutex> access_output_queue_(&output_queue_mutex_);
+      base::MutexGuard access_output_queue_(&output_queue_mutex_);
       if (output_queue_.empty()) return;
       job = output_queue_.front();
       output_queue_.pop();
@@ -140,9 +159,9 @@ void OptimizingCompileDispatcher::FlushOutputQueue(bool restore_function_code) {
 void OptimizingCompileDispatcher::Flush(BlockingBehavior blocking_behavior) {
   if (blocking_behavior == BlockingBehavior::kDontBlock) {
     if (FLAG_block_concurrent_recompilation) Unblock();
-    base::LockGuard<base::Mutex> access_input_queue_(&input_queue_mutex_);
+    base::MutexGuard access_input_queue_(&input_queue_mutex_);
     while (input_queue_length_ > 0) {
-      CompilationJob* job = input_queue_[InputQueueIndex(0)];
+      OptimizedCompilationJob* job = input_queue_[InputQueueIndex(0)];
       DCHECK_NOT_NULL(job);
       input_queue_shift_ = InputQueueIndex(1);
       input_queue_length_--;
@@ -154,12 +173,12 @@ void OptimizingCompileDispatcher::Flush(BlockingBehavior blocking_behavior) {
     }
     return;
   }
-  base::Release_Store(&mode_, static_cast<base::AtomicWord>(FLUSH));
+  mode_ = FLUSH;
   if (FLAG_block_concurrent_recompilation) Unblock();
   {
-    base::LockGuard<base::Mutex> lock_guard(&ref_count_mutex_);
+    base::MutexGuard lock_guard(&ref_count_mutex_);
     while (ref_count_ > 0) ref_count_zero_.Wait(&ref_count_mutex_);
-    base::Release_Store(&mode_, static_cast<base::AtomicWord>(COMPILE));
+    mode_ = COMPILE;
   }
   FlushOutputQueue(true);
   if (FLAG_trace_concurrent_recompilation) {
@@ -168,38 +187,34 @@ void OptimizingCompileDispatcher::Flush(BlockingBehavior blocking_behavior) {
 }
 
 void OptimizingCompileDispatcher::Stop() {
-  base::Release_Store(&mode_, static_cast<base::AtomicWord>(FLUSH));
+  mode_ = FLUSH;
   if (FLAG_block_concurrent_recompilation) Unblock();
   {
-    base::LockGuard<base::Mutex> lock_guard(&ref_count_mutex_);
+    base::MutexGuard lock_guard(&ref_count_mutex_);
     while (ref_count_ > 0) ref_count_zero_.Wait(&ref_count_mutex_);
-    base::Release_Store(&mode_, static_cast<base::AtomicWord>(COMPILE));
+    mode_ = COMPILE;
   }
 
-  if (recompilation_delay_ != 0) {
-    // At this point the optimizing compiler thread's event loop has stopped.
-    // There is no need for a mutex when reading input_queue_length_.
-    while (input_queue_length_ > 0) CompileNext(NextInput());
-    InstallOptimizedFunctions();
-  } else {
-    FlushOutputQueue(false);
-  }
+  // At this point the optimizing compiler thread's event loop has stopped.
+  // There is no need for a mutex when reading input_queue_length_.
+  DCHECK_EQ(input_queue_length_, 0);
+  FlushOutputQueue(false);
 }
 
 void OptimizingCompileDispatcher::InstallOptimizedFunctions() {
   HandleScope handle_scope(isolate_);
 
   for (;;) {
-    CompilationJob* job = nullptr;
+    OptimizedCompilationJob* job = nullptr;
     {
-      base::LockGuard<base::Mutex> access_output_queue_(&output_queue_mutex_);
+      base::MutexGuard access_output_queue_(&output_queue_mutex_);
       if (output_queue_.empty()) return;
       job = output_queue_.front();
       output_queue_.pop();
     }
-    CompilationInfo* info = job->compilation_info();
-    Handle<JSFunction> function(*info->closure());
-    if (function->HasOptimizedCode()) {
+    OptimizedCompilationInfo* info = job->compilation_info();
+    Handle<JSFunction> function(*info->closure(), isolate_);
+    if (function->HasAvailableOptimizedCode()) {
       if (FLAG_trace_concurrent_recompilation) {
         PrintF("  ** Aborting compilation for ");
         function->ShortPrint();
@@ -207,16 +222,17 @@ void OptimizingCompileDispatcher::InstallOptimizedFunctions() {
       }
       DisposeCompilationJob(job, false);
     } else {
-      Compiler::FinalizeCompilationJob(job, isolate_);
+      Compiler::FinalizeOptimizedCompilationJob(job, isolate_);
     }
   }
 }
 
-void OptimizingCompileDispatcher::QueueForOptimization(CompilationJob* job) {
+void OptimizingCompileDispatcher::QueueForOptimization(
+    OptimizedCompilationJob* job) {
   DCHECK(IsQueueAvailable());
   {
     // Add job to the back of the input queue.
-    base::LockGuard<base::Mutex> access_input_queue(&input_queue_mutex_);
+    base::MutexGuard access_input_queue(&input_queue_mutex_);
     DCHECK_LT(input_queue_length_, input_queue_capacity_);
     input_queue_[InputQueueIndex(input_queue_length_)] = job;
     input_queue_length_++;
@@ -224,15 +240,15 @@ void OptimizingCompileDispatcher::QueueForOptimization(CompilationJob* job) {
   if (FLAG_block_concurrent_recompilation) {
     blocked_jobs_++;
   } else {
-    V8::GetCurrentPlatform()->CallOnBackgroundThread(
-        new CompileTask(isolate_, this), v8::Platform::kShortRunningTask);
+    V8::GetCurrentPlatform()->CallOnWorkerThread(
+        std::make_unique<CompileTask>(isolate_, this));
   }
 }
 
 void OptimizingCompileDispatcher::Unblock() {
   while (blocked_jobs_ > 0) {
-    V8::GetCurrentPlatform()->CallOnBackgroundThread(
-        new CompileTask(isolate_, this), v8::Platform::kShortRunningTask);
+    V8::GetCurrentPlatform()->CallOnWorkerThread(
+        std::make_unique<CompileTask>(isolate_, this));
     blocked_jobs_--;
   }
 }
