@@ -181,6 +181,7 @@ function ServerResponse(req) {
 
   if (req.method === 'HEAD') this._hasBody = false;
 
+  this.req = req;
   this.sendDate = true;
   this._sent100 = false;
   this._expect_continue = false;
@@ -581,11 +582,7 @@ function socketOnTimeout() {
 
 function socketOnClose(socket, state) {
   debug('server socket close');
-  // Mark this parser as reusable
-  if (socket.parser) {
-    freeParser(socket.parser, null, socket);
-  }
-
+  freeParser(socket.parser, null, socket);
   abortIncoming(state.incoming);
 }
 
@@ -609,18 +606,15 @@ function socketOnEnd(server, socket, parser, state) {
 
   if (ret instanceof Error) {
     debug('parse error');
+    // socketOnError has additional logic and will call socket.destroy(err).
     FunctionPrototypeCall(socketOnError, socket, ret);
-    return;
-  }
-
-  if (!server.httpAllowHalfOpen) {
-    abortIncoming(state.incoming);
-    if (socket.writable) socket.end();
+  } else if (!server.httpAllowHalfOpen) {
+    socket.end();
   } else if (state.outgoing.length) {
     state.outgoing[state.outgoing.length - 1]._last = true;
   } else if (socket._httpMessage) {
     socket._httpMessage._last = true;
-  } else if (socket.writable) {
+  } else {
     socket.end();
   }
 }
@@ -635,6 +629,7 @@ function socketOnData(server, socket, parser, state, d) {
 
 function onRequestTimeout(socket) {
   socket[kRequestTimeout] = undefined;
+  // socketOnError has additional logic and will call socket.destroy(err).
   ReflectApply(socketOnError, socket, [new ERR_HTTP_REQUEST_TIMEOUT()]);
 }
 
