@@ -1458,7 +1458,8 @@ Server.prototype[EE.captureRejectionSymbol] = function(
 function SNICallback(servername, callback) {
   const contexts = this.server._contexts;
 
-  for (const elem of contexts) {
+  for (let i = contexts.length - 1; i >= 0; --i) {
+    const elem = contexts[i];
     if (RegExpPrototypeTest(elem[0], servername)) {
       callback(null, elem[1]);
       return;
