@@ -100,6 +100,7 @@ const {
   hideStackFrames
 } = require('internal/errors');
 const {
+  validateArray,
   validateBuffer,
   validateInteger,
   validateString
@@ -115,6 +116,10 @@ const {
   createUnsafeBuffer
 } = require('internal/buffer');
 
+const {
+  Blob,
+} = require('internal/blob');
+
 FastBuffer.prototype.constructor = Buffer;
 Buffer.prototype = FastBuffer.prototype;
 addBufferPrototypeMethods(Buffer.prototype);
@@ -530,9 +535,7 @@ Buffer.isEncoding = function isEncoding(encoding) {
 Buffer[kIsEncodingSymbol] = Buffer.isEncoding;
 
 Buffer.concat = function concat(list, length) {
-  if (!ArrayIsArray(list)) {
-    throw new ERR_INVALID_ARG_TYPE('list', 'Array', list);
-  }
+  validateArray(list, 'list');
 
   if (list.length === 0)
     return new FastBuffer();
@@ -646,6 +649,20 @@ const encodingOps = {
                     encodingsMap.base64,
                     dir)
   },
+  base64url: {
+    encoding: 'base64url',
+    encodingVal: encodingsMap.base64url,
+    byteLength: (string) => base64ByteLength(string, string.length),
+    write: (buf, string, offset, len) =>
+      buf.base64urlWrite(string, offset, len),
+    slice: (buf, start, end) => buf.base64urlSlice(start, end),
+    indexOf: (buf, val, byteOffset, dir) =>
+      indexOfBuffer(buf,
+                    fromStringFast(val, encodingOps.base64url),
+                    byteOffset,
+                    encodingsMap.base64url,
+                    dir)
+  },
   hex: {
     encoding: 'hex',
     encodingVal: encodingsMap.hex,
@@ -702,6 +719,11 @@ function getEncodingOps(encoding) {
       if (encoding === 'hex' || StringPrototypeToLowerCase(encoding) === 'hex')
         return encodingOps.hex;
       break;
+    case 9:
+      if (encoding === 'base64url' ||
+          StringPrototypeToLowerCase(encoding) === 'base64url')
+        return encodingOps.base64url;
+      break;
   }
 }
 
@@ -1191,6 +1213,7 @@ if (internalBinding('config').hasIntl) {
 }
 
 module.exports = {
+  Blob,
   Buffer,
   SlowBuffer,
   transcode,
