@@ -736,7 +736,7 @@ async function once(emitter, name, options = {}) {
     }
     function abortListener() {
       eventTargetAgnosticRemoveListener(emitter, name, resolver);
-      eventTargetAgnosticRemoveListener(emitter, 'error', resolver);
+      eventTargetAgnosticRemoveListener(emitter, 'error', errorListener);
       reject(lazyDOMException('The operation was aborted', 'AbortError'));
     }
     if (signal != null) {
