@@ -29,7 +29,11 @@ const {
     ERR_MISSING_ARGS
   }
 } = require('internal/errors');
-const { validateString, validateOneOf } = require('internal/validators');
+const {
+  validateArray,
+  validateOneOf,
+  validateString,
+} = require('internal/validators');
 const EventEmitter = require('events');
 const net = require('net');
 const dgram = require('dgram');
@@ -377,12 +381,12 @@ ChildProcess.prototype.spawn = function(options) {
   validateString(options.file, 'options.file');
   this.spawnfile = options.file;
 
-  if (ArrayIsArray(options.args))
-    this.spawnargs = options.args;
-  else if (options.args === undefined)
+  if (options.args === undefined) {
     this.spawnargs = [];
-  else
-    throw new ERR_INVALID_ARG_TYPE('options.args', 'Array', options.args);
+  } else {
+    validateArray(options.args, 'options.args');
+    this.spawnargs = options.args;
+  }
 
   const err = this._handle.spawn(options);
 
