@@ -6,21 +6,35 @@
 const {
   ArrayFrom,
   ArrayIsArray,
+  ArrayPrototypeForEach,
+  ArrayPrototypePush,
+  ArrayPrototypeUnshift,
   Boolean,
   ErrorCaptureStackTrace,
-  Map,
+  FunctionPrototypeBind,
   MathFloor,
   Number,
+  NumberPrototypeToFixed,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectKeys,
   ObjectPrototypeHasOwnProperty,
   ObjectValues,
+  ReflectApply,
+  ReflectConstruct,
   ReflectOwnKeys,
+  SafeArrayIterator,
+  SafeMap,
+  SafeWeakMap,
+  StringPrototypeIncludes,
+  StringPrototypePadStart,
+  StringPrototypeRepeat,
+  StringPrototypeReplace,
+  StringPrototypeSlice,
+  StringPrototypeSplit,
   Symbol,
   SymbolHasInstance,
   SymbolToStringTag,
-  WeakMap,
 } = primordials;
 
 const { trace } = internalBinding('trace_events');
@@ -80,14 +94,14 @@ const kBindStreamsLazy = Symbol('kBindStreamsLazy');
 const kUseStdout = Symbol('kUseStdout');
 const kUseStderr = Symbol('kUseStderr');
 
-const optionsMap = new WeakMap();
+const optionsMap = new SafeWeakMap();
 
 function Console(options /* or: stdout, stderr, ignoreErrors = true */) {
   // We have to test new.target here to see if this function is called
   // with new, because we need to define a custom instanceof to accommodate
   // the global console.
   if (!new.target) {
-    return new Console(...arguments);
+    return ReflectConstruct(Console, arguments);
   }
 
   if (!options || typeof options.write === 'function') {
@@ -137,16 +151,15 @@ function Console(options /* or: stdout, stderr, ignoreErrors = true */) {
   }
 
   // Bind the prototype functions to this Console instance
-  const keys = ObjectKeys(Console.prototype);
-  for (const key of keys) {
+  ArrayPrototypeForEach(ObjectKeys(Console.prototype), (key) => {
     // We have to bind the methods grabbed from the instance instead of from
     // the prototype so that users extending the Console can override them
     // from the prototype chain of the subclass.
-    this[key] = this[key].bind(this);
+    this[key] = FunctionPrototypeBind(this[key], this);
     ObjectDefineProperty(this[key], 'name', {
       value: key
     });
-  }
+  });
 
   this[kBindStreamsEager](stdout, stderr);
   this[kBindProperties](ignoreErrors, colorMode, groupIndentation);
@@ -224,9 +237,9 @@ ObjectDefineProperties(Console.prototype, {
           ...consolePropAttributes,
           value: Boolean(ignoreErrors)
         },
-        '_times': { ...consolePropAttributes, value: new Map() },
+        '_times': { ...consolePropAttributes, value: new SafeMap() },
         // Corresponds to https://console.spec.whatwg.org/#count-map
-        [kCounts]: { ...consolePropAttributes, value: new Map() },
+        [kCounts]: { ...consolePropAttributes, value: new SafeMap() },
         [kColorMode]: { ...consolePropAttributes, value: colorMode },
         [kIsConsole]: { ...consolePropAttributes, value: true },
         [kGroupIndent]: { ...consolePropAttributes, value: '' },
@@ -255,8 +268,8 @@ ObjectDefineProperties(Console.prototype, {
         this._stdoutErrorHandler : this._stderrErrorHandler;
 
       if (groupIndent.length !== 0) {
-        if (string.includes('\n')) {
-          string = string.replace(/\n/g, `\n${groupIndent}`);
+        if (StringPrototypeIncludes(string, '\n')) {
+          string = StringPrototypeReplace(string, /\n/g, `\n${groupIndent}`);
         }
         string = groupIndent + string;
       }
@@ -310,14 +323,16 @@ ObjectDefineProperties(Console.prototype, {
     ...consolePropAttributes,
     value: function(args) {
       const opts = this[kGetInspectOptions](this._stdout);
-      return formatWithOptions(opts, ...args);
+      ArrayPrototypeUnshift(args, opts);
+      return ReflectApply(formatWithOptions, null, args);
     }
   },
   [kFormatForStderr]: {
     ...consolePropAttributes,
     value: function(args) {
       const opts = this[kGetInspectOptions](this._stderr);
-      return formatWithOptions(opts, ...args);
+      ArrayPrototypeUnshift(args, opts);
+      return ReflectApply(formatWithOptions, null, args);
     }
   },
 });
@@ -402,7 +417,8 @@ const consoleMethods = {
   assert(expression, ...args) {
     if (!expression) {
       args[0] = `Assertion failed${args.length === 0 ? '' : `: ${args[0]}`}`;
-      this.warn(...args);  // The arguments will be formatted in warn() again
+      // The arguments will be formatted in warn() again
+      ReflectApply(this.warn, this, args);
     }
   },
 
@@ -448,15 +464,18 @@ const consoleMethods = {
 
   group(...data) {
     if (data.length > 0) {
-      this.log(...data);
+      ReflectApply(this.log, this, data);
     }
-    this[kGroupIndent] += ' '.repeat(this[kGroupIndentationWidth]);
+    this[kGroupIndent] +=
+      StringPrototypeRepeat(' ', this[kGroupIndentationWidth]);
   },
 
   groupEnd() {
-    this[kGroupIndent] =
-      this[kGroupIndent].slice(0, this[kGroupIndent].length -
-        this[kGroupIndentationWidth]);
+    this[kGroupIndent] = StringPrototypeSlice(
+      this[kGroupIndent],
+      0,
+      this[kGroupIndent].length - this[kGroupIndentationWidth]
+    );
   },
 
   // https://console.spec.whatwg.org/#table
@@ -501,14 +520,14 @@ const consoleMethods = {
       let length = 0;
       if (mapIter) {
         for (; i < tabularData.length / 2; ++i) {
-          keys.push(_inspect(tabularData[i * 2]));
-          values.push(_inspect(tabularData[i * 2 + 1]));
+          ArrayPrototypePush(keys, _inspect(tabularData[i * 2]));
+          ArrayPrototypePush(values, _inspect(tabularData[i * 2 + 1]));
           length++;
         }
       } else {
         for (const [k, v] of tabularData) {
-          keys.push(_inspect(k));
-          values.push(_inspect(v));
+          ArrayPrototypePush(keys, _inspect(k));
+          ArrayPrototypePush(values, _inspect(v));
           length++;
         }
       }
@@ -530,7 +549,7 @@ const consoleMethods = {
       const values = [];
       let length = 0;
       for (const v of tabularData) {
-        values.push(_inspect(v));
+        ArrayPrototypePush(values, _inspect(v));
         length++;
       }
       return final([iterKey, valuesKey], [getIndexArray(length), values]);
@@ -565,11 +584,11 @@ const consoleMethods = {
     const keys = ObjectKeys(map);
     const values = ObjectValues(map);
     if (hasPrimitives) {
-      keys.push(valuesKey);
-      values.push(valuesKeyArray);
+      ArrayPrototypePush(keys, valuesKey);
+      ArrayPrototypePush(values, valuesKeyArray);
     }
-    keys.unshift(indexKey);
-    values.unshift(indexKeyArray);
+    ArrayPrototypeUnshift(keys, indexKey);
+    ArrayPrototypeUnshift(values, indexKeyArray);
 
     return final(keys, values);
   },
@@ -590,13 +609,13 @@ function timeLogImpl(self, name, label, data) {
   if (data === undefined) {
     self.log('%s: %s', label, formatted);
   } else {
-    self.log('%s: %s', label, formatted, ...data);
+    self.log('%s: %s', label, formatted, ...new SafeArrayIterator(data));
   }
   return true;
 }
 
 function pad(value) {
-  return `${value}`.padStart(2, '0');
+  return StringPrototypePadStart(`${value}`, 2, '0');
 }
 
 function formatTime(ms) {
@@ -617,16 +636,19 @@ function formatTime(ms) {
   }
 
   if (hours !== 0 || minutes !== 0) {
-    [seconds, ms] = seconds.toFixed(3).split('.');
+    ({ 0: seconds, 1: ms } = StringPrototypeSplit(
+      NumberPrototypeToFixed(seconds, 3),
+      '.'
+    ));
     const res = hours !== 0 ? `${hours}:${pad(minutes)}` : minutes;
     return `${res}:${pad(seconds)}.${ms} (${hours !== 0 ? 'h:m' : ''}m:ss.mmm)`;
   }
 
   if (seconds !== 0) {
-    return `${seconds.toFixed(3)}s`;
+    return `${NumberPrototypeToFixed(seconds, 3)}s`;
   }
 
-  return `${Number(ms.toFixed(3))}ms`;
+  return `${Number(NumberPrototypeToFixed(ms, 3))}ms`;
 }
 
 const keyKey = 'Key';
