@@ -1,7 +1,6 @@
 'use strict';
 
 const {
-  ArrayIsArray,
   ArrayPrototypeForEach,
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
@@ -14,7 +13,7 @@ const {
 
 const errors = require('internal/errors');
 const { isIP } = require('internal/net');
-const { validateInt32 } = require('internal/validators');
+const { validateArray, validateInt32 } = require('internal/validators');
 const {
   ChannelWrap,
   strerror,
@@ -60,9 +59,7 @@ class Resolver {
   }
 
   setServers(servers) {
-    if (!ArrayIsArray(servers)) {
-      throw new ERR_INVALID_ARG_TYPE('servers', 'Array', servers);
-    }
+    validateArray(servers, 'servers');
 
     // Cache the original servers because in the event of an error while
     // setting the servers, c-ares won't have any servers available for
