@@ -104,13 +104,8 @@ function getConditionsSet(conditions) {
 const realpathCache = new SafeMap();
 const packageJSONCache = new SafeMap();  /* string -> PackageConfig */
 
-function tryStatSync(path) {
-  try {
-    return statSync(path);
-  } catch {
-    return new Stats();
-  }
-}
+const tryStatSync =
+  (path) => statSync(path, { throwIfNoEntry: false }) ?? new Stats();
 
 function getPackageConfig(path, specifier, base) {
   const existing = packageJSONCache.get(path);
