@@ -5,7 +5,6 @@
 // thread and the worker threads.
 
 const {
-  ArrayIsArray,
   ArrayPrototypeMap,
   ArrayPrototypePush,
   ArrayPrototypeSplice,
@@ -35,6 +34,7 @@ const {
   }
 } = require('internal/errors');
 const format = require('internal/util/inspect').format;
+const { validateArray } = require('internal/validators');
 const constants = internalBinding('constants').os.signals;
 
 function assert(x, msg) {
@@ -55,9 +55,7 @@ function getFastAPIs(binding) {
     _hrtime.hrtime();
 
     if (time !== undefined) {
-      if (!ArrayIsArray(time)) {
-        throw new ERR_INVALID_ARG_TYPE('time', 'Array', time);
-      }
+      validateArray(time, 'time');
       if (time.length !== 2) {
         throw new ERR_OUT_OF_RANGE('time', 2, time.length);
       }
