@@ -15,10 +15,6 @@ const {
   enqueueMicrotask
 } = internalBinding('task_queue');
 
-const {
-  triggerUncaughtException
-} = internalBinding('errors');
-
 const {
   setHasRejectionToWarn,
   hasRejectionToWarn,
@@ -38,11 +34,12 @@ const {
   symbols: { async_id_symbol, trigger_async_id_symbol }
 } = require('internal/async_hooks');
 const {
-  ERR_INVALID_CALLBACK,
   ERR_INVALID_ARG_TYPE
 } = require('internal/errors').codes;
 const FixedQueue = require('internal/fixed_queue');
 
+const { validateCallback } = require('internal/validators');
+
 // *Must* match Environment::TickInfo::Fields in src/env.h.
 const kHasTickScheduled = 0;
 
@@ -71,7 +68,7 @@ function processTicksAndRejections() {
   do {
     while (tock = queue.shift()) {
       const asyncId = tock[async_id_symbol];
-      emitBefore(asyncId, tock[trigger_async_id_symbol]);
+      emitBefore(asyncId, tock[trigger_async_id_symbol], tock);
 
       try {
         const callback = tock.callback;
@@ -103,8 +100,7 @@ function processTicksAndRejections() {
 // `nextTick()` will not enqueue any callback when the process is about to
 // exit since the callback would not have a chance to be executed.
 function nextTick(callback) {
-  if (typeof callback !== 'function')
-    throw new ERR_INVALID_CALLBACK(callback);
+  validateCallback(callback);
 
   if (process._exiting)
     return;
@@ -151,10 +147,6 @@ function runMicrotask() {
     const callback = this.callback;
     try {
       callback();
-    } catch (error) {
-      // runInAsyncScope() swallows the error so we need to catch
-      // it and handle it here.
-      triggerUncaughtException(error, false /* fromPromise */);
     } finally {
       this.emitDestroy();
     }
