@@ -27,6 +27,7 @@
 
 const {
   FunctionPrototype,
+  Error,
   ObjectDefineProperty,
   ObjectDefineProperties,
   ObjectSetPrototypeOf,
@@ -41,6 +42,11 @@ const EE = require('events');
 const Stream = require('internal/streams/legacy').Stream;
 const { Buffer } = require('buffer');
 const destroyImpl = require('internal/streams/destroy');
+
+const {
+  addAbortSignalNoValidate,
+} = require('internal/streams/add-abort-signal');
+
 const {
   getHighWaterMark,
   getDefaultHighWaterMark
@@ -263,6 +269,8 @@ function Writable(options) {
 
     if (typeof options.construct === 'function')
       this._construct = options.construct;
+    if (options.signal)
+      addAbortSignalNoValidate(options.signal, this);
   }
 
   Stream.call(this, options);
@@ -283,8 +291,8 @@ Writable.prototype.pipe = function() {
   errorOrDestroy(this, new ERR_STREAM_CANNOT_PIPE());
 };
 
-Writable.prototype.write = function(chunk, encoding, cb) {
-  const state = this._writableState;
+function _write(stream, chunk, encoding, cb) {
+  const state = stream._writableState;
 
   if (typeof encoding === 'function') {
     cb = encoding;
@@ -326,11 +334,15 @@ Writable.prototype.write = function(chunk, encoding, cb) {
 
   if (err) {
     process.nextTick(cb, err);
-    errorOrDestroy(this, err, true);
-    return false;
+    errorOrDestroy(stream, err, true);
+    return err;
   }
   state.pendingcb++;
-  return writeOrBuffer(this, state, chunk, encoding, cb);
+  return writeOrBuffer(stream, state, chunk, encoding, cb);
+}
+
+Writable.prototype.write = function(chunk, encoding, cb) {
+  return _write(this, chunk, encoding, cb) === true;
 };
 
 Writable.prototype.cork = function() {
@@ -438,7 +450,7 @@ function onwrite(stream, er) {
 
   if (er) {
     // Avoid V8 leak, https://github.com/nodejs/node/pull/34103#issuecomment-652002364
-    er.stack;
+    er.stack; // eslint-disable-line no-unused-expressions
 
     if (!state.errored) {
       state.errored = er;
@@ -600,8 +612,14 @@ Writable.prototype.end = function(chunk, encoding, cb) {
     encoding = null;
   }
 
-  if (chunk !== null && chunk !== undefined)
-    this.write(chunk, encoding);
+  let err;
+
+  if (chunk !== null && chunk !== undefined) {
+    const ret = _write(this, chunk, encoding);
+    if (ret instanceof Error) {
+      err = ret;
+    }
+  }
 
   // .end() fully uncorks.
   if (state.corked) {
@@ -609,12 +627,15 @@ Writable.prototype.end = function(chunk, encoding, cb) {
     this.uncork();
   }
 
-  // This is forgiving in terms of unnecessary calls to end() and can hide
-  // logic errors. However, usually such errors are harmless and causing a
-  // hard error can be disproportionately destructive. It is not always
-  // trivial for the user to determine whether end() needs to be called or not.
-  let err;
-  if (!state.errored && !state.ending) {
+  if (err) {
+    // Do nothing...
+  } else if (!state.errored && !state.ending) {
+    // This is forgiving in terms of unnecessary calls to end() and can hide
+    // logic errors. However, usually such errors are harmless and causing a
+    // hard error can be disproportionately destructive. It is not always
+    // trivial for the user to determine whether end() needs to be called
+    // or not.
+
     state.ending = true;
     finishMaybe(this, state, true);
     state.ended = true;
