@@ -108,10 +108,12 @@ const trigger_async_id_symbol = Symbol('triggerId');
 const kHasPrimitive = Symbol('kHasPrimitive');
 
 const {
-  ERR_INVALID_CALLBACK,
   ERR_OUT_OF_RANGE
 } = require('internal/errors').codes;
-const { validateNumber } = require('internal/validators');
+const {
+  validateCallback,
+  validateNumber,
+} = require('internal/validators');
 
 const L = require('internal/linkedlist');
 const PriorityQueue = require('internal/priority_queue');
@@ -368,9 +370,7 @@ function insert(item, msecs, start = getLibuvNow()) {
 
 function setUnrefTimeout(callback, after) {
   // Type checking identical to setTimeout()
-  if (typeof callback !== 'function') {
-    throw new ERR_INVALID_CALLBACK(callback);
-  }
+  validateCallback(callback);
 
   const timer = new Timeout(callback, after, undefined, false, false);
   insert(timer, timer._idleTimeout);
