@@ -180,6 +180,11 @@ function slowCases(enc) {
         StringPrototypeToLowerCase(`${enc}`) === 'utf-16le')
         return 'utf16le';
       break;
+    case 9:
+      if (enc === 'base64url' || enc === 'BASE64URL' ||
+          StringPrototypeToLowerCase(`${enc}`) === 'base64url')
+        return 'base64url';
+      break;
     default:
       if (enc === '') return 'utf8';
   }
