@@ -2,11 +2,12 @@
 
 const {
   ArrayIsArray,
+  ArrayPrototypeFilter,
+  ArrayPrototypePush,
   BigIntPrototypeValueOf,
   BooleanPrototypeValueOf,
   DatePrototypeGetTime,
   Error,
-  Map,
   NumberIsNaN,
   NumberPrototypeValueOf,
   ObjectGetOwnPropertySymbols,
@@ -16,10 +17,11 @@ const {
   ObjectPrototypeHasOwnProperty,
   ObjectPrototypePropertyIsEnumerable,
   ObjectPrototypeToString,
-  Set,
+  SafeMap,
+  SafeSet,
   StringPrototypeValueOf,
   SymbolPrototypeValueOf,
-  SymbolToStringTag,
+  TypedArrayPrototypeGetSymbolToStringTag,
   Uint8Array,
 } = primordials;
 
@@ -42,15 +44,6 @@ const {
   isSymbolObject,
   isFloat32Array,
   isFloat64Array,
-  isUint8Array,
-  isUint8ClampedArray,
-  isUint16Array,
-  isUint32Array,
-  isInt8Array,
-  isInt16Array,
-  isInt32Array,
-  isBigInt64Array,
-  isBigUint64Array
 } = types;
 const {
   getOwnNonIndexProperties,
@@ -124,37 +117,6 @@ function isEqualBoxedPrimitive(val1, val2) {
   assert.fail(`Unknown boxed type ${val1}`);
 }
 
-function isIdenticalTypedArrayType(a, b) {
-  // Fast path to reduce type checks in the common case.
-  const check = types[`is${a[SymbolToStringTag]}`];
-  if (check !== undefined && check(a)) {
-    return check(b);
-  }
-  // Manipulated Symbol.toStringTag.
-  for (const check of [
-    isUint16Array,
-    isUint32Array,
-    isInt8Array,
-    isInt16Array,
-    isInt32Array,
-    isFloat32Array,
-    isFloat64Array,
-    isBigInt64Array,
-    isBigUint64Array,
-    isUint8ClampedArray,
-    isUint8Array,
-  ]) {
-    if (check(a)) {
-      return check(b);
-    }
-  }
-  /* c8 ignore next 4 */
-  assert.fail(
-    `Unknown TypedArray type checking ${a[SymbolToStringTag]} ${a}\n` +
-    `and ${b[SymbolToStringTag]} ${b}`
-  );
-}
-
 // Notes: Type tags are historical [[Class]] properties that can be set by
 // FunctionTemplate::SetClassName() in C++ or Symbol.toStringTag in JS
 // and retrieved using Object.prototype.toString.call(obj) in JS
@@ -238,8 +200,10 @@ function innerDeepEqual(val1, val2, strict, memos) {
       return false;
     }
   } else if (isArrayBufferView(val1)) {
-    if (!isIdenticalTypedArrayType(val1, val2))
+    if (TypedArrayPrototypeGetSymbolToStringTag(val1) !==
+        TypedArrayPrototypeGetSymbolToStringTag(val2)) {
       return false;
+    }
     if (!strict && (isFloat32Array(val1) || isFloat64Array(val1))) {
       if (!areSimilarFloatArrays(val1, val2)) {
         return false;
@@ -291,7 +255,10 @@ function innerDeepEqual(val1, val2, strict, memos) {
 }
 
 function getEnumerables(val, keys) {
-  return keys.filter((k) => ObjectPrototypePropertyIsEnumerable(val, k));
+  return ArrayPrototypeFilter(
+    keys,
+    (k) => ObjectPrototypePropertyIsEnumerable(val, k)
+  );
 }
 
 function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
@@ -315,7 +282,7 @@ function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
   // Cheap key test
   let i = 0;
   for (; i < aKeys.length; i++) {
-    if (!ObjectPrototypePropertyIsEnumerable(val2, aKeys[i])) {
+    if (!ObjectPrototypeHasOwnProperty(val2, aKeys[i])) {
       return false;
     }
   }
@@ -330,7 +297,7 @@ function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
           if (!ObjectPrototypePropertyIsEnumerable(val2, key)) {
             return false;
           }
-          aKeys.push(key);
+          ArrayPrototypePush(aKeys, key);
           count++;
         } else if (ObjectPrototypePropertyIsEnumerable(val2, key)) {
           return false;
@@ -360,8 +327,8 @@ function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
   // Use memos to handle cycles.
   if (memos === undefined) {
     memos = {
-      val1: new Map(),
-      val2: new Map(),
+      val1: new SafeMap(),
+      val2: new SafeMap(),
       position: 0
     };
   } else {
@@ -458,11 +425,11 @@ function setEquiv(a, b, strict, memo) {
     // to check this improves the worst case scenario instead.
     if (typeof val === 'object' && val !== null) {
       if (set === null) {
-        set = new Set();
+        set = new SafeSet();
       }
-      // If the specified value doesn't exist in the second set it's a non-null
+      // If the specified value doesn't exist in the second set its an not null
       // object (or non strict only: a not matching primitive) we'll need to go
-      // hunting for something that's deep-(strict-)equal to it. To make this
+      // hunting for something thats deep-(strict-)equal to it. To make this
       // O(n log n) complexity we have to copy these values in a new set first.
       set.add(val);
     } else if (!b.has(val)) {
@@ -475,7 +442,7 @@ function setEquiv(a, b, strict, memo) {
       }
 
       if (set === null) {
-        set = new Set();
+        set = new SafeSet();
       }
       set.add(val);
     }
@@ -518,10 +485,10 @@ function mapHasEqualEntry(set, map, key1, item1, strict, memo) {
 function mapEquiv(a, b, strict, memo) {
   let set = null;
 
-  for (const { 0: key, 1: item1 } of a) {
+  for (const [key, item1] of a) {
     if (typeof key === 'object' && key !== null) {
       if (set === null) {
-        set = new Set();
+        set = new SafeSet();
       }
       set.add(key);
     } else {
@@ -537,7 +504,7 @@ function mapEquiv(a, b, strict, memo) {
         if (!mapMightHaveLoosePrim(a, b, key, item1, memo))
           return false;
         if (set === null) {
-          set = new Set();
+          set = new SafeSet();
         }
         set.add(key);
       }
@@ -545,7 +512,7 @@ function mapEquiv(a, b, strict, memo) {
   }
 
   if (set !== null) {
-    for (const { 0: key, 1: item } of b) {
+    for (const [key, item] of b) {
       if (typeof key === 'object' && key !== null) {
         if (!mapHasEqualEntry(set, a, key, item, strict, memo))
           return false;
