@@ -1,12 +1,15 @@
 'use strict';
 
 const {
+  FunctionPrototype,
   FunctionPrototypeBind,
   ObjectCreate,
   ObjectDefineProperty,
   RegExp,
   RegExpPrototypeTest,
-  StringPrototypeToUpperCase
+  SafeArrayIterator,
+  StringPrototypeToLowerCase,
+  StringPrototypeToUpperCase,
 } = primordials;
 
 const { inspect, format, formatWithOptions } = require('internal/util/inspect');
@@ -36,13 +39,13 @@ function initializeDebugEnv(debugEnv) {
 function emitWarningIfNeeded(set) {
   if ('HTTP' === set || 'HTTP2' === set) {
     process.emitWarning('Setting the NODE_DEBUG environment variable ' +
-      'to \'' + set.toLowerCase() + '\' can expose sensitive ' +
+      'to \'' + StringPrototypeToLowerCase(set) + '\' can expose sensitive ' +
       'data (such as passwords, tokens and authentication headers) ' +
       'in the resulting log.');
   }
 }
 
-function noop() {}
+const noop = FunctionPrototype;
 
 function debuglogImpl(enabled, set) {
   if (debugImpls[set] === undefined) {
@@ -78,7 +81,7 @@ function debuglog(set, cb) {
     debug = debuglogImpl(enabled, set);
     if (typeof cb === 'function')
       cb(debug);
-    debug(...args);
+    debug(...new SafeArrayIterator(args));
   };
   let enabled;
   let test = () => {
@@ -86,7 +89,7 @@ function debuglog(set, cb) {
     test = () => enabled;
     return enabled;
   };
-  const logger = (...args) => debug(...args);
+  const logger = (...args) => debug(...new SafeArrayIterator(args));
   ObjectDefineProperty(logger, 'enabled', {
     get() {
       return test();
