@@ -43,6 +43,7 @@ const {
   ObjectPrototypePropertyIsEnumerable,
   ObjectSeal,
   ObjectSetPrototypeOf,
+  ReflectOwnKeys,
   RegExp,
   RegExpPrototypeTest,
   RegExpPrototypeToString,
@@ -309,7 +310,8 @@ function inspect(value, opts) {
       ctx.showHidden = opts;
     } else if (opts) {
       const optKeys = ObjectKeys(opts);
-      for (const key of optKeys) {
+      for (let i = 0; i < optKeys.length; ++i) {
+        const key = optKeys[i];
         // TODO(BridgeAR): Find a solution what to do about stylize. Either make
         // this function public or add a new API with a similar or better
         // functionality.
@@ -614,11 +616,7 @@ function addPrototypeProperties(ctx, main, obj, recurseTimes, output) {
       ArrayPrototypeForEach(keys, (key) => keySet.add(key));
     }
     // Get all own property names and symbols.
-    keys = ObjectGetOwnPropertyNames(obj);
-    const symbols = ObjectGetOwnPropertySymbols(obj);
-    if (symbols.length !== 0) {
-      ArrayPrototypePush(keys, ...symbols);
-    }
+    keys = ReflectOwnKeys(obj);
     for (const key of keys) {
       // Ignore the `constructor` property and keys that exist on layers above.
       if (key === 'constructor' ||
@@ -666,7 +664,7 @@ function getKeys(value, showHidden) {
   if (showHidden) {
     keys = ObjectGetOwnPropertyNames(value);
     if (symbols.length !== 0)
-      keys.push(...symbols);
+      ArrayPrototypePush(keys, ...symbols);
   } else {
     // This might throw if `value` is a Module Namespace Object from an
     // unevaluated module, but we don't want to perform the actual type
@@ -1869,7 +1867,7 @@ function tryStringify(arg) {
 }
 
 function format(...args) {
-  return formatWithOptionsInternal(undefined, ...args);
+  return formatWithOptionsInternal(undefined, args);
 }
 
 function formatWithOptions(inspectOptions, ...args) {
@@ -1877,10 +1875,10 @@ function formatWithOptions(inspectOptions, ...args) {
     throw new ERR_INVALID_ARG_TYPE(
       'inspectOptions', 'object', inspectOptions);
   }
-  return formatWithOptionsInternal(inspectOptions, ...args);
+  return formatWithOptionsInternal(inspectOptions, args);
 }
 
-function formatWithOptionsInternal(inspectOptions, ...args) {
+function formatWithOptionsInternal(inspectOptions, args) {
   const first = args[0];
   let a = 0;
   let str = '';
