@@ -4,6 +4,7 @@
 
 const {
   ArrayIsArray,
+  ArrayPrototypeForEach,
   ArrayPrototypeMap,
   ArrayPrototypePush,
   Float64Array,
@@ -14,7 +15,9 @@ const {
   ObjectEntries,
   Promise,
   PromiseResolve,
+  ReflectApply,
   RegExpPrototypeTest,
+  SafeArrayIterator,
   String,
   Symbol,
   SymbolFor,
@@ -54,6 +57,7 @@ const {
 } = workerIo;
 const { deserializeError } = require('internal/error_serdes');
 const { fileURLToPath, isURLInstance, pathToFileURL } = require('internal/url');
+const { validateArray } = require('internal/validators');
 
 const {
   ownsProcessState,
@@ -106,9 +110,7 @@ class Worker extends EventEmitter {
     }
     let argv;
     if (options.argv) {
-      if (!ArrayIsArray(options.argv)) {
-        throw new ERR_INVALID_ARG_TYPE('options.argv', 'Array', options.argv);
-      }
+      validateArray(options.argv, 'options.argv');
       argv = ArrayPrototypeMap(options.argv, String);
     }
 
@@ -155,8 +157,10 @@ class Worker extends EventEmitter {
     let env;
     if (typeof options.env === 'object' && options.env !== null) {
       env = ObjectCreate(null);
-      for (const [ key, value ] of ObjectEntries(options.env))
-        env[key] = `${value}`;
+      ArrayPrototypeForEach(
+        ObjectEntries(options.env),
+        ({ 0: key, 1: value }) => { env[key] = `${value}`; }
+      );
     } else if (options.env == null) {
       env = process.env;
     } else if (options.env !== SHARE_ENV) {
@@ -209,12 +213,13 @@ class Worker extends EventEmitter {
     const transferList = [port2];
     // If transferList is provided.
     if (options.transferList)
-      ArrayPrototypePush(transferList, ...options.transferList);
+      ArrayPrototypePush(transferList,
+                         ...new SafeArrayIterator(options.transferList));
 
     this[kPublicPort] = port1;
-    for (const event of ['message', 'messageerror']) {
+    ArrayPrototypeForEach(['message', 'messageerror'], (event) => {
       this[kPublicPort].on(event, (message) => this.emit(event, message));
-    }
+    });
     setupPortReferencing(this[kPublicPort], this, 'message');
     this[kPort].postMessage({
       argv,
@@ -279,8 +284,9 @@ class Worker extends EventEmitter {
       {
         const { stream, chunks } = message;
         const readable = this[kParentSideStdio][stream];
-        for (const { chunk, encoding } of chunks)
+        ArrayPrototypeForEach(chunks, ({ chunk, encoding }) => {
           readable.push(chunk, encoding);
+        });
         return;
       }
       case messageTypes.STDIO_WANTS_MORE_DATA:
@@ -314,7 +320,7 @@ class Worker extends EventEmitter {
   postMessage(...args) {
     if (this[kPublicPort] === null) return;
 
-    this[kPublicPort].postMessage(...args);
+    ReflectApply(this[kPublicPort].postMessage, this[kPublicPort], args);
   }
 
   terminate(callback) {
