@@ -82,7 +82,6 @@ const kInsertEntry = Symbol('insert-entry');
 const kGetEntries = Symbol('get-entries');
 const kIndex = Symbol('index');
 const kMarks = Symbol('marks');
-const kCount = Symbol('count');
 
 const observers = {};
 const observerableTypes = [
@@ -287,11 +286,6 @@ class PerformanceObserverEntryList {
         writable: true,
         enumerable: false,
         value: {}
-      },
-      [kCount]: {
-        writable: true,
-        enumerable: false,
-        value: 0
       }
     });
     L.init(this[kEntries]);
@@ -300,11 +294,6 @@ class PerformanceObserverEntryList {
   [kInsertEntry](entry) {
     const item = { entry };
     L.append(this[kEntries], item);
-    this[kCount]++;
-  }
-
-  get length() {
-    return this[kCount];
   }
 
   [kGetEntries](name, type) {
