@@ -59,14 +59,15 @@ const {
   StringPrototypeTrim,
   Symbol,
   SymbolAsyncIterator,
+  SafeStringIterator,
 } = primordials;
 
 const {
   ERR_INVALID_ARG_VALUE,
-  ERR_INVALID_CALLBACK,
   ERR_INVALID_CURSOR_POS,
 } = require('internal/errors').codes;
 const {
+  validateCallback,
   validateString,
   validateUint32,
 } = require('internal/validators');
@@ -273,9 +274,6 @@ function Interface(input, output, completer, terminal) {
     input.on('keypress', onkeypress);
     input.on('end', ontermend);
 
-    // Current line
-    this.line = '';
-
     this._setRawMode(true);
     this.terminal = true;
 
@@ -291,6 +289,9 @@ function Interface(input, output, completer, terminal) {
     self.once('close', onSelfCloseWithTerminal);
   }
 
+  // Current line
+  this.line = '';
+
   input.resume();
 }
 
@@ -752,7 +753,7 @@ Interface.prototype._getDisplayPos = function(str) {
   const col = this.columns;
   let rows = 0;
   str = stripVTControlCharacters(str);
-  for (const char of str) {
+  for (const char of new SafeStringIterator(str)) {
     if (char === '\n') {
       // Rows must be incremented by 1 even if offset = 0 or col = +Infinity.
       rows += MathCeil(offset / col) || 1;
@@ -1168,7 +1169,7 @@ function emitKeypressEvents(stream, iface = {}) {
         iface.isCompletionEnabled = false;
 
         let length = 0;
-        for (const character of string) {
+        for (const character of new SafeStringIterator(string)) {
           length += character.length;
           if (length === string.length) {
             iface.isCompletionEnabled = true;
@@ -1215,8 +1216,9 @@ function emitKeypressEvents(stream, iface = {}) {
  */
 
 function cursorTo(stream, x, y, callback) {
-  if (callback !== undefined && typeof callback !== 'function')
-    throw new ERR_INVALID_CALLBACK(callback);
+  if (callback !== undefined) {
+    validateCallback(callback);
+  }
 
   if (typeof y === 'function') {
     callback = y;
@@ -1241,8 +1243,9 @@ function cursorTo(stream, x, y, callback) {
  */
 
 function moveCursor(stream, dx, dy, callback) {
-  if (callback !== undefined && typeof callback !== 'function')
-    throw new ERR_INVALID_CALLBACK(callback);
+  if (callback !== undefined) {
+    validateCallback(callback);
+  }
 
   if (stream == null || !(dx || dy)) {
     if (typeof callback === 'function')
@@ -1275,8 +1278,9 @@ function moveCursor(stream, dx, dy, callback) {
  */
 
 function clearLine(stream, dir, callback) {
-  if (callback !== undefined && typeof callback !== 'function')
-    throw new ERR_INVALID_CALLBACK(callback);
+  if (callback !== undefined) {
+    validateCallback(callback);
+  }
 
   if (stream === null || stream === undefined) {
     if (typeof callback === 'function')
@@ -1297,8 +1301,9 @@ function clearLine(stream, dir, callback) {
  */
 
 function clearScreenDown(stream, callback) {
-  if (callback !== undefined && typeof callback !== 'function')
-    throw new ERR_INVALID_CALLBACK(callback);
+  if (callback !== undefined) {
+    validateCallback(callback);
+  }
 
   if (stream === null || stream === undefined) {
     if (typeof callback === 'function')
