@@ -18,6 +18,8 @@ const {
   codes: { ERR_INVALID_ARG_TYPE }
 } = require('internal/errors');
 
+const { validateAbortSignal } = require('internal/validators');
+
 function cancelListenerHandler(clear, reject) {
   if (!this._destroyed) {
     clear(this);
@@ -35,15 +37,10 @@ function setTimeout(after, value, options = {}) {
         options));
   }
   const { signal, ref = true } = options;
-  if (signal !== undefined &&
-      (signal === null ||
-       typeof signal !== 'object' ||
-       !('aborted' in signal))) {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options.signal',
-        'AbortSignal',
-        signal));
+  try {
+    validateAbortSignal(signal, 'options.signal');
+  } catch (err) {
+    return PromiseReject(err);
   }
   if (typeof ref !== 'boolean') {
     return PromiseReject(
@@ -52,10 +49,7 @@ function setTimeout(after, value, options = {}) {
         'boolean',
         ref));
   }
-  // TODO(@jasnell): If a decision is made that this cannot be backported
-  // to 12.x, then this can be converted to use optional chaining to
-  // simplify the check.
-  if (signal && signal.aborted) {
+  if (signal?.aborted) {
     return PromiseReject(new AbortError());
   }
   let oncancel;
@@ -85,15 +79,10 @@ function setImmediate(value, options = {}) {
         options));
   }
   const { signal, ref = true } = options;
-  if (signal !== undefined &&
-    (signal === null ||
-     typeof signal !== 'object' ||
-     !('aborted' in signal))) {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options.signal',
-        'AbortSignal',
-        signal));
+  try {
+    validateAbortSignal(signal, 'options.signal');
+  } catch (err) {
+    return PromiseReject(err);
   }
   if (typeof ref !== 'boolean') {
     return PromiseReject(
@@ -102,10 +91,7 @@ function setImmediate(value, options = {}) {
         'boolean',
         ref));
   }
-  // TODO(@jasnell): If a decision is made that this cannot be backported
-  // to 12.x, then this can be converted to use optional chaining to
-  // simplify the check.
-  if (signal && signal.aborted) {
+  if (signal?.aborted) {
     return PromiseReject(new AbortError());
   }
   let oncancel;
