@@ -23,6 +23,7 @@
 
 const {
   ArrayBuffer,
+  ArrayPrototypeForEach,
   ArrayPrototypeMap,
   ArrayPrototypePush,
   Error,
@@ -803,8 +804,8 @@ function Brotli(opts, mode) {
   assert(mode === BROTLI_DECODE || mode === BROTLI_ENCODE);
 
   TypedArrayPrototypeFill(brotliInitParamsArray, -1);
-  if (opts && opts.params) {
-    for (const origKey of ObjectKeys(opts.params)) {
+  if (opts?.params) {
+    ArrayPrototypeForEach(ObjectKeys(opts.params), (origKey) => {
       const key = +origKey;
       if (NumberIsNaN(key) || key < 0 || key > kMaxBrotliParam ||
           (brotliInitParamsArray[key] | 0) !== -1) {
@@ -817,7 +818,7 @@ function Brotli(opts, mode) {
                                        'number', opts.params[origKey]);
       }
       brotliInitParamsArray[key] = value;
-    }
+    });
   }
 
   const handle = mode === BROTLI_DECODE ?
