@@ -31,13 +31,13 @@ for (const link of toc.match(/<a.*?>/g)) {
   const data = fs.readFileSync(source + '/' + href, 'utf8');
 
   // Split the doc.
-  const match = /(<\/ul>\s*)?<\/div>\s*<div id="apicontent">/.exec(data);
+  const match = /(<\/ul>\s*)?<\/\w+>\s*<\w+ id="apicontent">/.exec(data);
 
   contents += data.slice(0, match.index)
-    .replace(/[\s\S]*?<div id="toc">\s*<h2>.*?<\/h2>\s*(<ul>\s*)?/, '');
+    .replace(/[\s\S]*?id="toc"[^>]*>\s*<\w+>.*?<\/\w+>\s*(<ul>\s*)?/, '');
 
   apicontent += data.slice(match.index + match[0].length)
-    .replace(/(<\/div>\s*)*\s*<script[\s\S]*/, '')
+    .replace(/<!-- API END -->[\s\S]*/, '')
     .replace(/<a href="(\w[^#"]*)#/g, (match, href) => {
       return htmlFiles.includes(href) ? '<a href="#' : match;
     })
@@ -49,7 +49,7 @@ for (const link of toc.match(/<a.*?>/g)) {
 
 // Replace various mentions of index with all.
 let all = toc.replace(/index\.html/g, 'all.html')
-  .replace('<a href="all.html" name="toc">', '<a href="index.html" name="toc">')
+  .replace('<a href="all.html">', '<a href="index.html">')
   .replace('index.json', 'all.json')
   .replace('api-section-index', 'api-section-all')
   .replace('data-id="index"', 'data-id="all"')
@@ -59,17 +59,30 @@ let all = toc.replace(/index\.html/g, 'all.html')
 all = all.replace(/<title>.*?\| /, '<title>');
 
 // Insert the combined table of contents.
-const tocStart = /<div id="toc">\s*<h2>.*?<\/h2>\s*/.exec(all);
+const tocStart = /<\w+ id="toc"[^>]*>\s*<\w+>.*?<\/\w+>\s*/.exec(all);
 all = all.slice(0, tocStart.index + tocStart[0].length) +
   '<ul>\n' + contents + '</ul>\n' +
   all.slice(tocStart.index + tocStart[0].length);
 
 // Replace apicontent with the concatenated set of apicontents from each source.
-const apiStart = /<div id="apicontent">\s*/.exec(all);
-const apiEnd = /(\s*<\/div>)*\s*<script /.exec(all);
+const apiStart = /<\w+ id="apicontent">\s*/.exec(all);
+const apiEnd = all.lastIndexOf('<!-- API END -->');
 all = all.slice(0, apiStart.index + apiStart[0].length) +
   apicontent +
-  all.slice(apiEnd.index);
+  all.slice(apiEnd);
 
 // Write results.
 fs.writeFileSync(source + '/all.html', all, 'utf8');
+
+// Validate all hrefs have a target.
+const ids = new Set();
+const idRe = / id="(\w+)"/g;
+let match;
+while (match = idRe.exec(all)) {
+  ids.add(match[1]);
+}
+
+const hrefRe = / href="#(\w+)"/g;
+while (match = hrefRe.exec(all)) {
+  if (!ids.has(match[1])) throw new Error(`link not found: ${match[1]}`);
+}
